/******************************************************************************
* DSGE_MH.G
* generates draws from the posterior distribution
* via the random walk Metropolis-Hastings algorithm
*
* upadted:  06/16/2005 
******************************************************************************/

/** set seed **/
/** rndseed 1234; **/


/** start log file **/
outstr = opath $+ "/" $+ ftos(timedt,"%*.*lf",1,0) $+ "mh.log";
output file = ^outstr reset;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = specname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** parameter specification: transformation and bounds **/
_trspec      = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;
_bounds      = _trspec[.,2 3];


/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = lpath $+ "/pm" $+ pmsel;
lretc = lpath $+ "/pmret" $+ pmsel;

/** check posterior mode results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para  = para';          /** paraest was saved as row vector in _pm.g **/
npara = rows(para);

open fhretc = ^lretc for read;
_csretcode = readr(fhretc,1);
closeall fhretc;

/********************************************************
* Load multiplication matrix for parameters
********************************************************/
lmult = lpath $+ "/mult" $+ pmsel;       /** sigmult **/
lhess = lpath $+ "/hess" $+ pmsel;     /** HHM **/

/** check Hessian results **/
if not(exist(lmult $+ ".dat"));
    "";
    "Please run Hessian finding program first.";
    "";
    end;
endif;

/** open **/
open fhmult = ^lmult for read;
SIGSCALE    = readr( fhmult,npara );
closeall fhmult;

open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/** rank of Hessian (theoretical) **/
SIGPROPdim = sumc(pmaskinv);

/** inverse by Singular Value Decomposition (SVD) **/
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

/** log determinant for nonsingular part **/
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/********************************************************
* Initialize Output files
********************************************************/

opath   = opath $+ "/mhrun" $+ mhrun;
chkdir  = chk_dir(opath);

/** print specification **/

"";
"===================================";
"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"Scaling of Hess  " cc;
"";
"===================================";
"";

"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3]  =
                "-*.*s " 10 6
                "*.*lf " 10 4;
d = printfm(outmat,mask,fmt);
{lnpost0, lnpy0} = objfcn_mh(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";

statname = { POST LIKE PROP PROPPOST REJECT };

ostat = opath $+ "/st";
opara = opath $+ "/pa";

/** check if append is possible **/
if not(exist(opara $+ ".dat"));
    appendmode=0;
endif;


/*******************************
** Create new output files
*******************************/

"-----------------------------------------------------------";
"Creating new output files";
"Press Key to Continue";
"-----------------------------------------------------------";
"";

create fhstat = ^ostat with ^statname, 5, 4;
create fhpara = ^opara with ^para_names, npara, 4;


/********************************************************
* Metropolis-Hastings algorithm
********************************************************/


/** initialize PARANEW **/
/** trembles the posterior mode **/
valid = 0;
do until valid;
   paranew = para + cc0*(sigscale*rndn(npara,1));
   paranew = paranew.*pmaskinv + pfix.*pmask;

   {postnew, likenew} = objfcn_mh(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;
endo;

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if (iblock == 1);
      /** save initial value as 1st draw in 1st block **/
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
   else;
      /** don't save initial values **/
      j=1;
   endif;

   /** with-in block loop **/
    do until j > nsim;

      /** propose **/
      paranew = paraold + cc*(sigscale*rndn(npara,1));
      paranew = paranew.*pmaskinv + pfix.*pmask;

      /** evaluate at the candidate **/
      {postnew, likenew} = objfcn_mh(paranew);
      propdens = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                 -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
      propsim[j,1] = propdens;
      proppostsim[j,1] = postnew;

      /** accept/reject rule  **/
      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          postsim[j,1] = postnew;
          likesim[j,1] = likenew;
          parasim[j,.] = paranew';

          /** updates **/
          paraold = paranew;
          postold = postnew;
          likeold = likenew;

      else;

          /** reject proposed jump **/
          likesim[j,1] = likeold;
          postsim[j,1] = postold;
          parasim[j,.] = paraold';
          rej[j] = 1;

      endif;

      j = j+1;
    endo;               /** with-in block  **/

    " ";
    "Rejection: perct " meanc(rej);
    "Likelihood       " meanc(likesim);
    "Posterior        " meanc(postsim);
    "Parameters       "; meanc(parasim)';
    "";

   /** set finish time **/
   tfin = date;

   "Elapsed Time"; etstr(ethsec(tstart,tfin));
   "";

   /** save output **/
   wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;           /** block **/

closeall fhstat, fhpara;

/** close log file **/
output file = ^outstr off;



