/******************************************************************************
* NKMP_HESS.G
* computes the Hessian at the posterior mode
*
* upadted:  06/02/2005
******************************************************************************/


/********************************************************
* House cleaning and set defaults
********************************************************/

pmsel   = "";

/** Global defaults **/
__output = 0;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 0;       /** 1 for verbose **/
_fcmptol = 1E-10;

/** start log file **/
outstr = opath $+ "/" $+ ftos(timedt,"%*.*lf",1,0) $+ "hess.log";
output file = ^outstr reset;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = specname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

npara    = rows(prior);
pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = lpath $+ "/pm" $+ pmsel;
lretc = lpath $+ "/pmret" $+ pmsel;

/** check if pm results exist **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1)';
closeall fhpara;

"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3]  =
                "-*.*s " 10 6
                "*.*lf " 10 4;
d = printfm(outmat,mask,fmt);

"";
"Prior*Likelihood at Mode";
objfcn_hess(para);
"";

/********************************************************
* Compute Hessian:
* element by element, fine tune with dxscale
********************************************************/
comphess:

    /** dxscale **/
    ndx      = 6;                   /** number of grid **/
    dx       = exp(-seqa(5,2,ndx)); /** grid **/
    dxscale  = ones(npara,1);       /** scale can be set differently across parameters **/

    /** set storage **/
    hessian  = zeros(npara,npara);
    hessdiag = zeros(ndx,1);

    /** evaluate posteior at its mode **/
    fx = objfcn_hess(para);

    /** diagonal elements **/
    seli = 1;
    do until seli > npara;

        /** evaluation **/
        i=1;
        do until i > ndx;
            paradx = para;
            parady = para;
            paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];  /** forward **/
            parady[seli] = parady[seli] - dx[i]*dxscale[seli];  /** backward **/
            fdx = objfcn_hess(paradx);
            fdy = objfcn_hess(parady);
            hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2;
            i = i+1;
        endo;

        /** negative of Hessian (average of values at 3rd and 4th grid) **/
        hessian[seli,seli] = -0.5*(hessdiag[3]+hessdiag[4]);

        seli = seli+1;
    endo;

    /** off-diagonal elements                                   **/
    /** Make sure that correlations are between -1 and 1        **/
    /** errorij contains the index of elements that are invalid **/
    
    errorij = {};

    seli = 1; vidx = 1;
    do until seli > npara;

        selj = seli+1;          /** upper triangular part **/

        do until selj > npara;

            /** evaluation **/
            i=1;
            do until i > ndx;
                paradx = para;
                parady = para;
                paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
                parady[selj] = parady[selj] - dx[i]*dxscale[selj];
                paradxdy = paradx;
                paradxdy[selj] = paradxdy[selj] - dx[i]*dxscale[selj];
                fdx = objfcn_hess(paradx);
                fdy = objfcn_hess(parady);
                fdxdy = objfcn_hess(paradxdy);
                hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[selj]);
              i = i+1;
            endo;


            /** negative of Hessian (average of values at 3rd and 4th grid) **/
            hessian[seli,selj] = -0.5*(hessdiag[3]+hessdiag[4]);

            /** calculate correlation **/
            if ( hessian[seli,seli] == 0 ) or (hessian[selj,selj] == 0);        /** zero variance **/
                corrij = 0;
            else;                   /** (covariance)/(standard deviations) **/
                corrij = hessian[seli,selj]/sqrt(hessian[seli,seli]*hessian[selj,selj]);
            endif;

            /** adjust correlation **/
            if abs(corrij)>0.98;                    /** close to 1 **/
                hessian[seli,selj]=0.9*sqrt(hessian[seli,seli]*hessian[selj,selj]);
                errorij = errorij |(seli~selj~corrij);
            elseif abs(corrij)<0.005;           /** close to 0 **/
                hessian[seli,selj]=0;
            endif;

            /** symmetrize **/
            hessian[selj,seli] = hessian[seli,selj];

            selj=selj+1;
        endo;
        seli = seli+1;
    endo;

    hessian = real(hessian);

    /** Save output **/
    ohess = opath $+ "/hess" $+ pmsel;
    create fhhess=^ohess with hess, npara, 8;
    wr = writer(fhhess,hessian);
    closeall fhhess;

    /********************************************************
    * Compute covariance matrix and penalty (posterior mode)
    ********************************************************/

    /** load (negative) Hessian **/
    lhess = opath $+ "/hess" $+ pmsel;
    open fhhess = ^lhess for read;
    HHM = readr(fhhess,npara);
    closeall fhhess;

    /** rank of Hessian (theoretical) **/
    rankHHM = sumc(pmaskinv);

    /** inverse by Singular Value Decomposition (SVD) **/
    {u , s, v} = svd1(HHM);
    invHHMdet = 1;

    i = 1;
    do until i > npara;
       if i > rankHHM;
          s[i,i] = 0;
       else;
          s[i,i]    = 1/s[i,i];
          invHHMdet = invHHMdet*s[i,i];         /** determinant for nonsingular part **/
       endif;
       i = i+1;
    endo;

    invHHM  = u*s*u';           /** inverse of negative Hessian: covariance **/
    sigmult = u*sqrt(s);        /** square root of covariance matrix at posterior mode **/
    penalt  = (rankHHM/2)*ln(2*pi) + 0.5*ln(invHHMdet); /** posterior mode penalty **/

    /** print result **/
    "";"";
    "Determinant of variance matrix";
    invHHMdet;
    "";
    "s.e. at the posterior mode";
    sqrt(diag(invHHM));
    "";
    "Post Mode Penalty";
    penalt;
    "";
    
    pause(1E-5);

    /** Save sigmult (square root of posterior mode covariance) **/
    omult = opath $+ "/mult" $+pmsel;
    create fhmult=^omult with MULT, npara, 8;
    wr = writer(fhmult,sigmult);
    closeall fhmult;



